/***************************************************************************
                          bhv_player.c  -  description
                             -------------------
    begin                : Sat Apr 5 2003
    copyright            : (C) 2003 by Steven Goodwin
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <malloc.h>
#include <string.h>
#include "explore.h"
#include "level.h"
#include "render.h"
#include "objects.h"

#define K_PLY_INVINCIBILITY_TIMEOUT		50
#define K_PLAYERFALLJUMPSPEED	2
#define K_PLAYERCRAWLSPEED	3
#define K_PLAYERWALKSPEED	4
#define K_PLAYERJUMPSPEED	5

#define K_MOVE_NORMAL		0
#define K_MOVE_FALLING		1
#define K_MOVE_SLIDE_LEFT	2
#define K_MOVE_SLIDE_RIGHT	4	
#define K_MOVE_JUMPING		8

typedef enum  {
	eWalking,
	eJumping,
	eFalling,
	eDead,
	}  tPlyState;

typedef struct sPLY_STATE {
	int		iLives;
	int		iAirSupply;
	int		iScore;
	int		iKeysLeft;

	tPlyState	iState;

	int		iAnimFrame;
	int		iAnimOffset;

	int		iRespawnInvincibility;
	BOOL		bIsPlayerInvincible;

	} EX_PLY_STATE;

static void plyStart(EX_OBJECT *pObj);
static void plyReset(EX_OBJECT *pObj);
static void plyUpdate(EX_OBJECT *pObj, int frames);
static tObjType plyGetType(const EX_OBJECT *pObj);
static void plyDraw(const EX_OBJECT *pObj);
static void plyDestroy(EX_OBJECT *pObj);

static void plyChangeToState(const EX_OBJECT *pObj, tPlyState new_state, int iParam);


BOOL Bhv_Player(EX_OBJECT *pObj, const char *pParams)
{
	pObj->pGfx = TheGame.pPlayerGfx;
	pObj->VTable.Start = plyStart;
	pObj->VTable.Reset = plyReset;
	pObj->VTable.GetType = plyGetType;
	pObj->VTable.Update = plyUpdate;
	pObj->VTable.Draw = plyDraw;
	pObj->VTable.Destroy = plyDestroy;

	/* Parse params into our own private setup structure */
	pObj->pBhvStateData = malloc(sizeof(EX_PLY_STATE));
	if (!pObj->pBhvStateData)
		return FALSE;

return TRUE;
}

static void plyStart(EX_OBJECT *pObj)
{
EX_PLY_STATE *pState = pObj->pBhvStateData;

	plyReset(pObj);
	pState->iLives = 5;
	pState->iScore = 0;
}

static void plyReset(EX_OBJECT *pObj)
{
EX_PLY_STATE *pState = pObj->pBhvStateData;
EX_OBJECT *pObjList = TheGame.curr_level.pObjList;

	pObj->x = pObj->init_x;
	pObj->y = pObj->init_y;
	pState->iAirSupply = 1280;
	pState->iState = eWalking;
	pState->iAnimFrame = 0;
	pState->iAnimOffset = T_PLAYER_WALK_LEFT;
	pState->iRespawnInvincibility = 0;
	pState->bIsPlayerInvincible = TRUE;

	pState->iKeysLeft = 0;
	while(pObjList)
		{
		if (exGetTypeObject(pObjList) == eKey)
			{
			pState->iKeysLeft++;
			}

		pObjList=pObjList->pNext;
		}

}

static tObjType plyGetType(const EX_OBJECT *pObj)
{
	return ePlayer;
}

static void plyDestroy(EX_OBJECT *pObj)
{
EX_PLY_STATE *pState = pObj->pBhvStateData;

	free(pState);
}

static void plyUpdate(EX_OBJECT *pObj, int frames)
{
EX_PLY_STATE *pState = pObj->pBhvStateData;

	if (TheGame.iface.iMove[EXI_MOVE_LEFT])
		{
		pObj->x -= K_PLAYERWALKSPEED * frames;
		}

	if (TheGame.iface.iMove[EXI_MOVE_RIGHT])
		{
		pObj->x += K_PLAYERWALKSPEED * frames;
		}

	if (TheGame.iface.iMove[EXI_MOVE_JUMP])
		{
		pObj->y -= TheGame.iTileHeight/4;
		}

	if (TheGame.iface.iMove[EXI_MOVE_DOWN])
		{
		pObj->y += TheGame.iTileHeight/4;
		}


	if (pState->bIsPlayerInvincible)
		{
		pState->iRespawnInvincibility += frames;
		if (pState->iRespawnInvincibility >= K_PLY_INVINCIBILITY_TIMEOUT)
			pState->bIsPlayerInvincible = FALSE;
		}

	pState->iAirSupply -= frames;
	if (pState->iAirSupply < 0)
		{
		pState->iAirSupply = 0;
		/* Player is now dead! */
		}

}

static void plyDraw(const EX_OBJECT *pObj)
{
const int iFromAlpha = 64;
const int iToAlpha = 255;
EX_PLY_STATE *pState = pObj->pBhvStateData;
BOOL bDraw=TRUE;
int iFrame;
int i, x;

	iFrame = pState->iAnimFrame+pState->iAnimOffset;

	if (pState->iState == eDead)	
		{
		iFrame = pState->iAnimFrame;
		if (iFrame == T_PLAYER_DEATH_END)
			bDraw = FALSE;
		}

	if (pState->bIsPlayerInvincible)
		{
		Uint8 old_alpha = pObj->pGfx->format->alpha;
		SDL_SetAlpha(pObj->pGfx, SDL_SRCALPHA, iFromAlpha + ((iToAlpha-iFromAlpha)*pState->iRespawnInvincibility)/K_PLY_INVINCIBILITY_TIMEOUT);
		exDrawTile(pObj->pGfx, iFrame, pObj->x, pObj->y);
		SDL_SetAlpha(pObj->pGfx, SDL_SRCALPHA, old_alpha);
		exDrawTile(pObj->pGfx, T_PLAYER_INVINCIBLE+(4*pState->iRespawnInvincibility/K_PLY_INVINCIBILITY_TIMEOUT), pObj->x, pObj->y);
		}
      else if (bDraw)
		{
		exDrawTile(pObj->pGfx, iFrame, pObj->x, pObj->y);
		}


	/* Draw blank line at bottom for lives */
	for(i=0;i<TheGame.iNumTileWidth;i++)
		exDrawTile(TheGame.pGeneralTileGfx, 0, i*TheGame.iTileWidth, TheGame.iScreenHeight-TheGame.iTileHeight);

	/* Draw lives counter */
	for(i=0;i<pState->iLives;i++)
		exDrawTile(pObj->pGfx, 0, i*40+10, TheGame.iScreenHeight-TheGame.iTileHeight);

	/* Draw Score */
	exDrawLevelTile(0,0);	/* repair background */
	exDrawLevelTile(1,0);	/* repair background */
	exWriteNumber(pState->iScore, 2, 0,0);

	/* Draw air supply */
	x = TheGame.iScreenWidth - TheGame.iTileWidth*10;

	/* Repair background */
	for(i = x/TheGame.iTileWidth; i < TheGame.iScreenWidth; ++i)
		exDrawLevelTile(i, 0);

	exDrawTile(TheGame.pGeneralTileGfx, T_AIRSUPPLY, x, 0);
	exDrawTile(TheGame.pGeneralTileGfx, T_AIRSUPPLY+1, x+TheGame.iTileWidth, 0);

	exDrawArea(TheGame.pGeneralTileGfx,
		/*src*/12*TheGame.iTileWidth, 4*TheGame.iTileHeight, pState->iAirSupply/10, TheGame.iTileHeight,
		/*dest*/x+TheGame.iTileWidth*2, 0);

}

